# import beignet
# import torch
#
#
# def test_legendre_polynomial_weight():
#     torch.testing.assert_close(
#         beignet.legendre_polynomial_weight(
#             torch.linspace(-1, 1, 11),
#         ),
#         torch.tensor([1.0]),
#     )
